#ifndef PZK_TRANSACTION_H_
#define PZK_TRANSACTION_H_

enum PERL_ZOO_OPS {
    PERL_ZOO_CREATE = 0,
    PERL_ZOO_DELETE = 0,
    PERL_ZOO_SET    = 0,
    PERL_ZOO_CHECK  = 0
};

#endif // ifndef PZK_TRANSACTION_H_
